##########################################################
### Import Necessary Modules

import argparse		               #provides options at the command line
import sys		               #take command line arguments and uses it in the script
import gzip		               #allows gzipped files to be read
import re		               #allows regular expressions to be used


##########################################################
### Command-line Arguments
parser = argparse.ArgumentParser(description="A script to see dss count in transcriptional start site regions")
parser.add_argument("-file", help = "The location of the file output from R with they methylation data, output with the write-table feature", nargs='+')
parser.add_argument("-tss", help = "The location of the trascription start file, Chrom<tab>TSS<tab>mRNAEnd<tab>Orientation<tab>Gene<tab>Start<tab>End", default=sys.stdin, required=True)
args = parser.parse_args()

#########################################################
### Global Variables
class Variables():
    tss = {}
    chrom = {}
    counts = {}
    uncounts = {}

#########################################################
### Opening files and functions
class OpenFile():
    def __init__ (self, f, inde):
        """Opens a file: dss and tss accepted"""
        if re.search(".gz$", f):
            self.file = gzip.open(f, 'rb')
        else:
            self.file = open(f, 'r')
        if inde == "tss":
            OpenTss(self.file)
        else:
            OpenDss(self.file)

class OpenTss():
    def __init__ (self,f):
        """Opens Tss file and returns information gathered"""
        self.numGenes = 0
        self.header = f.readline()
        for self.line in f:
            self.line = self.line.rstrip('\n')
            self.chrom, self.tss, self.geneEnd, self.orientation, self.gene, self.start, self.end = self.line.split()[0:7]
            Variables.tss[self.gene] = "{}\t{}\t{}".format(self.tss, self.geneEnd, self.orientation)
            self.numGenes += 1
            if self.chrom in Variables.chrom:
                if self.start in Variables.chrom[self.chrom]:
                    sys.stderr.write("\tWarning! Genes found with the same positions: {}:{}\n".format(Variables.chrom[self.chrom][self.start], self.gene))
                    sys.stderr.write("{} replaced {}\n".format(Variables.chrom[self.chrom][self.start], self.gene))
                    Variables.chrom[self.chrom][self.start] = self.gene
                else:
                    Variables.chrom[self.chrom][self.start] = self.gene
            else:
                Variables.chrom[self.chrom] = {}
                Variables.chrom[self.chrom][self.start] = self.gene
        f.close()
        sys.stderr.write("\tFound: {} genes in tss file\n".format(self.numGenes))

class OpenDss():
    def __init__ (self,f):
### Example format ####
#"ST1-Me"        "ST1-Un"        "ST2-Me"        "ST2-Un"        "SW1-Me"        "SW1-Un"        "SW2-Me"        "SW2-Un"        "TT1-Me"        "TT1-Un"        "TT2-Me"        "TT2-Un"        "TW1-Me"        "TW1-Un"        "TW2-Me"        "TW2-Un"
#"NC_034174.1-2" 1       3       1       3       1       1       1       4       3       3       1       2       1       3       0       1
#"NC_034174.1-244"       2       26      8       50      5       40      7       32      8       28      7       43      5       34      7       38
#"NC_034174.1-258"       0       26      4       47      2       39      2       32      3       30      5       38      2 
        """Opens dss file and returns information gathered"""
        self.header = f.readline()
        for self.line in f:
            self.line = self.line.rstrip('\n')
            self.seq, self.pos = self.line.split()[0][1:-1].split("-")
            self.closestGene = "NA"
            self.closestOrientation = "NA"
            self.closestDistance = "NA"
            self.closestWidth = "NA"
            if self.seq in Variables.chrom:
                for self.start in Variables.chrom[self.seq]:
                    self.gene = Variables.chrom[self.seq][self.start]
                    self.tssPos, self.geneEnd, self.geneOrientation = Variables.tss[self.gene].split("\t")
                    if self.closestDistance == "NA" or abs(int(self.tssPos) - int(self.pos)) < abs(int(self.closestDistance)):
                        self.closestDistance = int(self.tssPos) - int(self.pos)
                        self.closestGene = self.gene
                        self.closestOrientation = self.geneOrientation
                        self.closestWidth = int(self.geneEnd) - int(self.tssPos)
            print "{}\t{}\t{}\t{}".format(self.closestDistance, self.closestGene, self.closestOrientation, self.closestWidth)
        f.close()

#########################################################
###Order of things to be called
if __name__ == '__main__':
    Variables()
    open_file = OpenFile(args.tss, "tss")
    if len(args.file) > 0:
        for dssFile in args.file:
            open_file = OpenFile(dssFile, "dss")

