##########################################################
### Import Necessary Modules

import argparse		               #provides options at the command line
import sys		               #take command line arguments and uses it in the script
import gzip		               #allows gzipped files to be read
import re		               #allows regular expressions to be used

##########################################################
### Command-line Arguments
parser = argparse.ArgumentParser(description="A script to retrieve the protein names for all the genes in a list of genes")
parser.add_argument("-gff", help = "The location of the gff file", default=sys.stdin, required=True)
parser.add_argument("-gene", help = "The location of the gene file (output by R)", default=sys.stdin, required=True)
args = parser.parse_args()

#########################################################
### Open file (object-oriented programming)

class GlobalVar():
	###Makes variables accessible to all functions###
	GeneNames = {}

class OpenFile():
    	### Opens the file and directs it to a line-reader and stores information into variables used later
	def __init__ (self, filename, ty, ty2):
		"""Opens the file (accepts gzipped files)"""
		if re.search(".gz$", filename):
			self.file = gzip.open(filename, 'rb')
		else:
			self.file = open(filename, 'r')
		if ty == "gff":
			sys.stderr.write("Reading gff {} file\n".format(ty2))
			self.readLinesGFF(self.file, ty2)
		elif ty == "gene":
			sys.stderr.write("Reading gene ({}) file\n".format(ty2))          
			self.readLinesGene(self.file, ty2)			

	def readLinesGFF(self, f, ty2):
		"""Reads the lines from gff file and outputs protein names"""
		self.rna_name_2_gene_name = {}
		self.filename = f
		self.type = ty2
		self.gene_count = 0
		self.linked_proteins = 0
		self.already_printed = {}
		self.already_printed2 = {}
		for line in self.filename:
			line = line.rstrip('\n')
			if not re.search("^#", line):
				self.list_of_columns = line.split("\t")
				###Finds the link between gene id and protein name (it can be tricky as unique gene names may be the same between species)
				if re.search("^gene", self.list_of_columns[2]) and re.search("protein_coding", self.list_of_columns[8]):
					self.gene_count += 1
					self.chrom = self.list_of_columns[0]
					self.start = self.list_of_columns[3]
					self.end = self.list_of_columns[4]
					self.gene_name = "NA"
					self.gene_id = "NA"
					self.gene_unique = "NA"
					###The GFF file does not contain uniform column 8's########
					###########################################################
					for self.info in self.list_of_columns[8].split(";"):
						if re.search("^ID=gene", self.info):
							self.gene_unique = self.info[3:]
						elif re.search("^Dbxref=GeneID:", self.info):
							self.gene_id = self.info[14:]
						elif re.search("^Name=", self.info):
							self.gene_name = self.info[5:]
					###########################################################
					if self.gene_name not in GlobalVar.GeneNames:
						continue
                                        #print "\t{}".format(self.gene_name)
					if self.gene_name == "NA" or self.gene_id == "NA" or self.gene_unique == "NA":
						#sys.stderr.write("Warning, gene at Chr {}, Pos {} {} not named correctly\n".format(self.chrom, self.start, self.end))
						self.warning = 0
				###The mRNA id is needed to link the protein id to the gene id
				elif re.search("^mRNA", self.list_of_columns[2]):
					self.rna_id = "NA"
					self.gene_unique = "NA"
					###########################################################
					for self.info in self.list_of_columns[8].split(";"):
						if re.search("^ID=rna", self.info):
							self.rna_id = self.info[3:]
						elif re.search("^Parent=gene", self.info):
							self.gene_unique = self.info[7:]
					###########################################################
					if self.rna_id == "NA" or self.gene_unique == "NA":
						#sys.stderr.write("Warning, RNA at Chr {}, Pos {} {} not named correctly\n".format(self.list_of_columns[0], self.list_of_columns[3], self.list_of_columns[4]))
						self.warning = 0
					if self.rna_id in self.rna_name_2_gene_name:
						#sys.stderr.write("Warning: RNA_id: {} found more than once\n".format(self.rna_id))
						#sys.stderr.write("Line: {}\n".format(line))
						self.warning = 0
					else:
						self.rna_name_2_gene_name[self.rna_id] = self.gene_unique
				###The CDS information contains the protein id
				elif re.search("^CDS", self.list_of_columns[2]):
					self.parent_rna = "NA"
					self.gene_id = "NA"
					self.protein_name = "NA"
					self.gene_name = "NA"
					self.product_name = "NA"
					self.gene_unique = "NA"
					###########################################################
					for self.info in self.list_of_columns[8].split(";"):
						if re.search("^Parent=rna", self.info):
							self.parent_rna = self.info[7:]
						elif re.search("^Dbxref=GeneID:", self.info):
							self.gene_id = self.info.split(",")[0][14:]
						elif re.search("^Name=", self.info):
							self.protein_name = self.info[5:]
						elif re.search("^gene=", self.info):
							self.gene_name = self.info[5:]
						elif re.search("^product=", self.info):
							self.product_name = self.info[8:]					
					###########################################################
					if self.parent_rna in self.rna_name_2_gene_name:
						self.gene_unique = self.rna_name_2_gene_name[self.parent_rna]
					if (self.parent_rna == "NA" or self.gene_id == "NA" or self.protein_name == "NA" or
						self.gene_name == "NA" or self.product_name == "NA" or self.gene_unique == "NA"):
						#sys.stderr.write("Warning, Protein at Chr {}, Pos {} {} not named correctly\n".format(self.list_of_columns[0], self.list_of_columns[3], self.list_of_columns[4]))
						self.warning = 0
					self.linked_proteins += 1
					if str(self.gene_name) in GlobalVar.GeneNames:
						self.matches = 0
						if self.protein_name not in self.already_printed:
							print "{}\t{}\t{}\t{}\t{}\t{}".format(self.protein_name, self.gene_name, "NA", "NA", "NA", self.product_name)
							self.already_printed[self.protein_name] = 1
									
	
		sys.stderr.write("\tFound {} genes in gff file\n\tFound {} linked proteins\n\n".format(self.gene_count, self.linked_proteins))
		self.filename.close() 

	def readLinesGene(self, f, ty2):
		"""Reads the gene file and outputs it to a global variable"""
		self.filename = f
		self.type = ty2
		self.header = self.filename.readline()
		for line in self.filename:
			line = line.rstrip('\n')
                        line = line.replace("\"", "")
			self.position, self.GeneName = line.split(",")[0:2]
                        GlobalVar.GeneNames[str(self.GeneName)] = 1
		self.filename.close()

if __name__ == '__main__':
	GlobalVar()
	OpenFile(args.gene, "gene", "Other")
	OpenFile(args.gff, "gff", "Salp")
