##########################################################
### Import Necessary Modules

import argparse		               #provides options at the command line
import sys		               #take command line arguments and uses it in the script
import gzip		               #allows gzipped files to be read
import re		               #allows regular expressions to be used

##########################################################
### Command-line Arguments
parser = argparse.ArgumentParser(description="A script to retrieve the transcriptional start sites, if multiple, then the first")
parser.add_argument("-gff", help = "The location of the gff file", default=sys.stdin, required=True)
parser.add_argument("-distBefore", help = "The distance before the transcriptional start site", default=1000)
parser.add_argument("-distAfter", help = "The distance before the transcriptional start site", default=500)
args = parser.parse_args()

#########################################################
### Open file (object-oriented programming)

class OpenFile():
	def __init__ (self, filename, ty):
		"""Opens the file (accepts gzipped files)"""
		if re.search(".gz$", filename):
			self.file = gzip.open(filename, 'rb')
		else:
			self.file = open(filename, 'r')
		if ty == "gff":
			sys.stderr.write("Reading gff file: {}\n".format(filename))
			self.readLinesGFF(self.file)		

	def readLinesGFF(self, f):
		"""Reads the lines from gff file and outputs protein names"""
		self.first = {}
		for line in f:
			line = line.rstrip('\n')
			if not re.search("^#", line):
				self.list_of_columns = line.split("\t")
				if re.search("^mRNA", self.list_of_columns[2]):
					self.gene = "NA"
					for self.tmp in self.list_of_columns[8].split(";"):
						if re.search("^gene=", self.tmp):
							self.gene = self.tmp[5:]
					if self.gene not in self.first:
						if self.list_of_columns[6] == "+":
							self.distB = int(self.list_of_columns[3]) - int(args.distBefore)
							if int(self.distB) < 1 :
								self.distB = 1
							self.distA = int(self.list_of_columns[3]) + int(args.distAfter)
						else:
							self.distA = int(self.list_of_columns[3]) + int(args.distBefore)
							self.distB = int(self.list_of_columns[3]) - int(args.distAfter)
							if int(self.distB) < 1 :
								self.distB = 1
						print "{}\t{}\t{}\t{}\t{}\t{}\t{}".format(self.list_of_columns[0], self.list_of_columns[3], self.list_of_columns[4], self.list_of_columns[6], self.gene, self.distB, self.distA)
						self.first[self.gene] = 1
		f.close() 


if __name__ == '__main__':
	OpenFile(args.gff, "gff")
