##########################################################
### Import Necessary Modules

import argparse		               #provides options at the command line
import sys		               #take command line arguments and uses it in the script
import gzip		               #allows gzipped files to be read
import re		               #allows regular expressions to be used

##########################################################
### Command-line Arguments
parser = argparse.ArgumentParser(description="A script to retrieve only unique annotations from a protein annotation")
parser.add_argument("-gff", help = "The location of the gff file", default=sys.stdin, required=True)
parser.add_argument("-annot", help = "The location of the annotation file (output by Blast2Go)", default=sys.stdin, required=True)
args = parser.parse_args()

#########################################################
### Open file (object-oriented programming)

class GlobalVar():
	###Makes variables accessible to all functions###
	annotations = {}
	proteinSubset = {}
	order = {}

class OpenFile():
    	### Opens the file and directs it to a line-reader and stores information into variables used later
	def __init__ (self, filename, ty, ty2):
		"""Opens the file (accepts gzipped files)"""
		if re.search(".gz$", filename):
			self.file = gzip.open(filename, 'rb')
		else:
			self.file = open(filename, 'r')
		if ty == "gff":
			sys.stderr.write("Reading gff {} file\n".format(ty2))
			self.readLinesGFF(self.file, ty2)
		elif ty == "annot":
			sys.stderr.write("Reading annot ({}) file\n".format(ty2))          
			self.readLinesAnnot(self.file, ty2)			

	def readLinesGFF(self, f, ty2):
		"""Reads the lines from gff file and outputs protein names"""
		self.genes = {}
		self.filename = f
		self.type = ty2
		for line in self.filename:
			line = line.rstrip('\n')
			if not re.search("^#", line):
				self.list_of_columns = line.split("\t")
				if re.search("^CDS", self.list_of_columns[2]):
					self.parent_rna = "NA"
					self.gene_id = "NA"
					self.protein_name = "NA"
					self.gene_name = "NA"
					self.product_name = "NA"
					self.gene_unique = "NA"
					###########################################################
					for self.info in self.list_of_columns[8].split(";"):
						if re.search("^Parent=rna", self.info):
							self.parent_rna = self.info[7:]
						elif re.search("^Dbxref=GeneID:", self.info):
							self.gene_id = self.info.split(",")[0][14:]
						elif re.search("^Name=", self.info):
							self.protein_name = self.info[5:]
						elif re.search("^gene=", self.info):
							self.gene_name = self.info[5:]
						elif re.search("^product=", self.info):
							self.product_name = self.info[8:]					
					###########################################################
					if self.gene_name not in self.genes:
						GlobalVar.proteinSubset[self.protein_name] = 1
						self.genes[self.gene_name] = 1

		self.filename.close() 

	def readLinesAnnot(self, f, ty2):
		"""Reads the gene file and outputs it to a global variable"""
		self.filename = f
		self.type = ty2
		self.position = 0
		for line in self.filename:
			line = line.rstrip('\n')
			self.protein = line.split("\t")[0]
			if self.protein in GlobalVar.annotations:
				GlobalVar.annotations[self.protein] += ";{}".format(line)
			else:
				GlobalVar.annotations[self.protein] = "{}".format(line)
				GlobalVar.order[self.position] = self.protein
				self.position += 1
		self.filename.close()

class Output():
	def __init__ (self):
		"""Outputs the data"""
		for self.position in sorted(GlobalVar.order):	
			if GlobalVar.order[self.position] in GlobalVar.proteinSubset:
				for self.line in GlobalVar.annotations[GlobalVar.order[self.position]].split(";"):
					print "{}".format(self.line)

if __name__ == '__main__':
	GlobalVar()
	OpenFile(args.annot, "annot", "Other")
	OpenFile(args.gff, "gff", "Salp")
	Output()
