##########################################################
### Import Necessary Modules

import argparse		               #provides options at the command line
import sys		               #take command line arguments and uses it in the script
import gzip		               #allows gzipped files to be read
import re		               #allows regular expressions to be used

##########################################################
### Command-line Arguments
parser = argparse.ArgumentParser(description="A script to retrieve only unique annotations from a protein annotation")
parser.add_argument("-gff", help = "The location of the gff file", default=sys.stdin, required=True)
parser.add_argument("-annot", help = "The location of the annotation file (output by Blast2Go)", default=sys.stdin, required=True)
parser.add_argument("-list", help = "The location of the list file (output by R)", default=sys.stdin, required=True)
args = parser.parse_args()

#########################################################
### Open file (object-oriented programming)

class GlobalVar():
	###Makes variables accessible to all functions###
	annotations = {}
	gene2Protein = {}
	protein2Gene = {}
	listProtein = {}

class OpenFile():
    	### Opens the file and directs it to a line-reader and stores information into variables used later
	def __init__ (self, filename, ty, ty2):
		"""Opens the file (accepts gzipped files)"""
		if re.search(".gz$", filename):
			self.file = gzip.open(filename, 'rb')
		else:
			self.file = open(filename, 'r')
		if ty == "gff":
			sys.stderr.write("Reading gff {} file\n".format(ty2))
			self.readLinesGFF(self.file, ty2)
		elif ty == "annot":
			sys.stderr.write("Reading annot ({}) file\n".format(ty2))          
			self.readLinesAnnot(self.file, ty2)
		elif ty == "list":
			sys.stderr.write("Reading list ({}) file\n".format(ty2))          
			self.readLinesList(self.file, ty2)			

	def readLinesGFF(self, f, ty2):
		"""Reads the lines from gff file and outputs protein names"""
		self.genes = {}
		self.filename = f
		self.type = ty2
		for line in self.filename:
			line = line.rstrip('\n')
			if not re.search("^#", line):
				self.list_of_columns = line.split("\t")
				if re.search("^CDS", self.list_of_columns[2]):
					self.parent_rna = "NA"
					self.gene_id = "NA"
					self.protein_name = "NA"
					self.gene_name = "NA"
					self.product_name = "NA"
					self.gene_unique = "NA"
					###########################################################
					for self.info in self.list_of_columns[8].split(";"):
						if re.search("^Parent=rna", self.info):
							self.parent_rna = self.info[7:]
						elif re.search("^Dbxref=GeneID:", self.info):
							self.gene_id = self.info.split(",")[0][14:]
						elif re.search("^Name=", self.info):
							self.protein_name = self.info[5:]
						elif re.search("^gene=", self.info):
							self.gene_name = self.info[5:]
						elif re.search("^product=", self.info):
							self.product_name = self.info[8:]					
					###########################################################
					GlobalVar.protein2Gene[self.protein_name] = self.gene_name
					if self.gene_name in GlobalVar.gene2Protein:
						GlobalVar.gene2Protein[self.gene_name][self.protein_name] = 1
						#print "Found2:{}\t{}".format(self.gene_name, self.protein_name)
					else:
						GlobalVar.gene2Protein[self.gene_name] = {}
						GlobalVar.gene2Protein[self.gene_name][self.protein_name] = 1
						#print "Found1:{}\t{}".format(self.gene_name, self.protein_name)

		self.filename.close() 

	def readLinesAnnot(self, f, ty2):
		"""Reads the annotation file and outputs it to a global variable"""
		self.filename = f
		self.type = ty2
		self.position = 0
		for line in self.filename:
			line = line.rstrip('\n')
			self.protein = line.split("\t")[0]
			GlobalVar.annotations[self.protein] = 1
		self.filename.close()

	def readLinesList(self, f, ty2):
		"""Reads the gene list file"""
		self.filename = f
		self.type = ty2
		self.position = 0
		for line in self.filename:
			line = line.rstrip('\n')
			self.protein = line.split("\t")[0]
			GlobalVar.listProtein[self.protein] = 1
			#print self.protein
		self.filename.close()

class Output():
	def __init__ (self):
		"""Outputs the data"""
		#GlobalVar.protein2Gene
		#GlobalVar.gene2Protein
		#GlobalVar.annotations[protein]
		#GlobalVar.listProtein[protein]	
		for self.protein1 in GlobalVar.listProtein:
			if self.protein1 in GlobalVar.annotations:
				print "{}".format(self.protein1)

if __name__ == '__main__':
	GlobalVar()
	OpenFile(args.gff, "gff", "Salp")
	OpenFile(args.annot, "annot", "Other")
	OpenFile(args.list, "list", "Salp")
	Output()
